#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2014 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Core.Networks;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using System.Drawing;

namespace HeuristicLab.Networks.Programmable {
  [Item("User-Defined Node", "A node of a network which can be defined and programmed by the user.")]
  [StorableClass]
  public sealed class UserDefinedNode : ProgrammableNode, IUserDefinedNode {
    public static new Image StaticItemImage {
      get { return HeuristicLab.Common.Resources.VSImageLibrary.Script; }
    }

    new public PortCollection Ports {
      get { return base.Ports; }
    }
    public override bool CanChangeCode {
      get { return true; }
    }

    [StorableConstructor]
    private UserDefinedNode(bool deserializing) : base(deserializing) { }
    private UserDefinedNode(UserDefinedNode original, Cloner cloner) : base(original, cloner) { }
    public UserDefinedNode() : base("UserDefinedNode") { }
    public UserDefinedNode(string name) : base(name) { }
    public UserDefinedNode(string name, string description) : base(name, description) { }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new UserDefinedNode(this, cloner);
    }

    public void Initialize() {
      CompiledNetworkItem.DeregisterEvents();
      CompiledNetworkItem.Initialize();
      CompiledNetworkItem.RegisterEvents();
    }
  }
}
