﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Globalization;
using System.Text;

namespace HeuristicLab.Problems.VehicleRouting.ProblemInstances {
  public class CVRPTWStopInsertionInfo : CVRPStopInsertionInfo {

    private double arrivalTime;

    public double ArrivalTime {
      get { return arrivalTime; }
    }

    private double leaveTime;

    public double LeaveTime {
      get { return leaveTime; }
    }

    private double spareTime;

    public double SpareTime {
      get { return spareTime; }
    }

    private double waitingTime;

    public double WaitingTime {
      get { return waitingTime; }
    }

    public CVRPTWStopInsertionInfo(int start, int end, double distance, double spareCapacity, double arrivalTime, double leaveTime, double spareTime, double waitingTime)
      : base(start, end, distance, spareCapacity) {
      this.arrivalTime = arrivalTime;
      this.leaveTime = leaveTime;
      this.spareTime = spareTime;
      this.waitingTime = waitingTime;
    }

    protected override void GetStopInsertionReport(StringBuilder builder) {
      base.GetStopInsertionReport(builder);
      builder.Append("SpareTime:\t");
      builder.AppendLine(SpareTime.ToString(CultureInfo.CurrentCulture));
      builder.Append("ArrivalTime:\t");
      builder.AppendLine(ArrivalTime.ToString(CultureInfo.CurrentCulture));
      builder.Append("WaitingTime:\t");
      builder.AppendLine(Math.Max(0, WaitingTime).ToString(CultureInfo.CurrentCulture));
      builder.Append("LeaveTime:\t");
      builder.AppendLine(LeaveTime.ToString(CultureInfo.CurrentCulture));
    }
  }

  public class CVRPTWTourInsertionInfo : CVRPTourInsertionInfo {
    private double tourStartTime;

    public double TourStartTime {
      get { return tourStartTime; }
    }

    public CVRPTWTourInsertionInfo(int vehicle, double capacity, double tourStartTime)
      : base(vehicle, capacity) {
      this.tourStartTime = tourStartTime;
    }

    protected override void GetTourInsertionReport(StringBuilder builder) {
      base.GetTourInsertionReport(builder);
      builder.Append("Tour start:\t");
      builder.AppendLine(TourStartTime.ToString(CultureInfo.CurrentCulture));
    }
  }

  public class CVRPTWEvaluation : CVRPEvaluation {
    public double Tardiness { get; set; }
    public double TravelTime { get; set; }

    protected override void GetEvaluationReport(StringBuilder builder) {
      base.GetEvaluationReport(builder);
      builder.Append("TravelTime:\t");
      builder.AppendLine(TravelTime.ToString(CultureInfo.CurrentCulture));
      builder.Append("Tardiness:\t");
      builder.AppendLine(Tardiness.ToString(CultureInfo.CurrentCulture));
    }
  }
}
